"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _child_process = require("child_process");

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _path = _interopRequireDefault(require("path"));

var _split = _interopRequireDefault(require("split2"));

var _logger = _interopRequireDefault(require("@wdio/logger"));

var _tcpPortUsed = _interopRequireDefault(require("tcp-port-used"));

var _webdriverio = require("webdriverio");

var _getFilePath = _interopRequireDefault(require("./utils/getFilePath"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const log = (0, _logger.default)('chromedriver');
const DEFAULT_LOG_FILENAME = 'wdio-chromedriver.log';
const POLL_INTERVAL = 100;
const POLL_TIMEOUT = 10000;
const DEFAULT_CONNECTION = {
  protocol: 'http',
  hostname: 'localhost',
  port: 9515,
  path: '/'
};

const isMultiremote = obj => typeof obj === 'object' && !Array.isArray(obj);

const isChrome = cap => cap.browserName && cap.browserName.toLowerCase() === 'chrome';

class ChromeDriverLauncher {
  constructor(options, capabilities, config) {
    this.options = {
      protocol: options.protocol || DEFAULT_CONNECTION.protocol,
      hostname: options.hostname || DEFAULT_CONNECTION.hostname,
      port: options.port || DEFAULT_CONNECTION.port,
      path: options.path || DEFAULT_CONNECTION.path
    };
    this.outputDir = options.outputDir || config.outputDir;
    this.logFileName = options.logFileName || DEFAULT_LOG_FILENAME;
    this.capabilities = capabilities;
    this.args = options.args || [];
    this.chromedriverCustomPath = options.chromedriverCustomPath;
  }

  async onPrepare() {
    this.args.forEach(argument => {
      if (argument.includes('--port')) {
        throw new Error('Argument "--port" already exists');
      }

      if (argument.includes('--url-base')) {
        throw new Error('Argument "--url-base" already exists');
      }
    });
    this.args.push(`--port=${this.options.port}`);
    this.args.push(`--url-base=${this.options.path}`);
    /**
     * update capability connection options to connect
     * to chromedriver
     */

    this._mapCapabilities();

    let command = this.chromedriverCustomPath ? _path.default.resolve(this.chromedriverCustomPath) : this._getChromedriverPath();
    log.info(`Start Chromedriver (${command}) with args ${this.args.join(' ')}`);

    if (!_fsExtra.default.existsSync(command)) {
      log.warn('Could not find chromedriver in default path: ', command);
      log.warn('Falling back to use global chromedriver bin');
      command = process && process.platform === 'win32' ? 'chromedriver.exe' : 'chromedriver';
    }
    /**
     * wait for port to be available before starting Chromedriver
     */


    try {
      await _tcpPortUsed.default.waitUntilFree(this.options.port, POLL_INTERVAL, POLL_TIMEOUT);
    } catch (err) {
      throw new _webdriverio.SevereServiceError(`Couldn't start Chromedriver: ${err.message}\n` + `Please check if port ${this.options.port} is in use!`);
    }

    this.process = (0, _child_process.spawn)(command, this.args);

    if (typeof this.outputDir === 'string') {
      this._redirectLogStream();
    } else {
      this.process.stdout.pipe((0, _split.default)()).on('data', log.info);
      this.process.stderr.pipe((0, _split.default)()).on('data', log.warn);
    }

    try {
      await _tcpPortUsed.default.waitUntilUsed(this.options.port, POLL_INTERVAL, POLL_TIMEOUT);
    } catch (err) {
      throw new _webdriverio.SevereServiceError(`Couldn't start Chromedriver: ${err.message}\n` + 'Chromedriver failed to start.');
    }

    process.on('exit', this.onComplete.bind(this));
    process.on('SIGINT', this.onComplete.bind(this));
    process.on('uncaughtException', this.onComplete.bind(this));
  }

  onComplete() {
    if (this.process) {
      this.process.kill();
    }
  }

  _redirectLogStream() {
    const logFile = (0, _getFilePath.default)(this.outputDir, this.logFileName); // ensure file & directory exists

    _fsExtra.default.ensureFileSync(logFile);

    const logStream = _fsExtra.default.createWriteStream(logFile, {
      flags: 'w'
    });

    this.process.stdout.pipe(logStream);
    this.process.stderr.pipe(logStream);
  }

  _mapCapabilities() {
    if (isMultiremote(this.capabilities)) {
      for (const cap in this.capabilities) {
        if (isChrome(this.capabilities[cap].capabilities)) {
          Object.assign(this.capabilities[cap], this.options);
        }
      }
    } else {
      for (const cap of this.capabilities) {
        if (isChrome(cap)) {
          Object.assign(cap, this.options);
        }
      }
    }
  }

  _getChromedriverPath() {
    try {
      return require('chromedriver').path;
    } catch (e) {
      log.error('Can\'t load chromedriver, please define "chromedriverCustomPath" property or install dependency via "npm install chromedriver --save-dev"');
      throw new _webdriverio.SevereServiceError(e.message);
    }
  }

}

exports.default = ChromeDriverLauncher;